lexer grammar Tql;
@members {
	private static final int STRING_MODE = 1;
	private static final int PROPERTY_IDENTIFIER_MODE = 2;
	private static final int CLASS_IDENTIFIER_MODE = 3;
	private static final int ALIAS_MODE = 4;
	
	private int currentMode = STRING_MODE;
	private int pastMode;

	private RecognitionException recognitionException;
	private String errorMessage;
	
	private void setMode(int mode)
	{
		pastMode = currentMode;
		currentMode = mode;
	}

	public void displayRecognitionError(String[] tokenNames, RecognitionException e)
	{
		if(!hasError())
		{
			setRecognitionException(e);
			setErrorMessage(getErrorMessage(e, tokenNames));		
		}
	}
	
	public RecognitionException getRecognitionException()
	{
		return recognitionException;
	}
	
	private void setRecognitionException(RecognitionException e)
	{
		recognitionException = e;
	}
	
	public boolean hasError()
	{
		return (getRecognitionException() != null);
	}
	
	public String getErrorMessage()
	{
		return errorMessage;
	}
	
	private void setErrorMessage(String error)
	{
		errorMessage = error;
	}
}
@header {
package com.compoundtheory.antlr;

/**
* Transfer Query Language Lexer
*
* @author Mark Mandel
*/
}

// $ANTLR src "T:\transfer-root\antlr/Tql.g" 264
SELECT	:
	{ currentMode == STRING_MODE }?=>
	'select'
	{ setMode(PROPERTY_IDENTIFIER_MODE); } 
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 269
FROM	:
	{ currentMode == STRING_MODE }?=>
	'from'
	{ setMode(CLASS_IDENTIFIER_MODE); }
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 274
WHERE	:
	{ currentMode == STRING_MODE }?=>
	'where'
	{ setMode(PROPERTY_IDENTIFIER_MODE); }
		;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 279
COMMA	:
	','
	{ setMode(pastMode); }
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 283
AS	: //special case, not worrying about past mode
	{ currentMode == STRING_MODE }?=>
	'as'
	{ currentMode = ALIAS_MODE; }
		;		
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 288
ALIAS	
	:
	{ currentMode == ALIAS_MODE }?=>
	LETTER(LETTER|DIGIT|'_')*
	{ currentMode = STRING_MODE; }
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 294
OPERATOR
	:
	('='|'>'|'<'|'!='|'<>'|'>='|'<='|'like')
	{ setMode(PROPERTY_IDENTIFIER_MODE); }
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 299
BOOLEAN_LOGIC	
	:
	{ currentMode == STRING_MODE }?=>
	( 'and' | 'or' )
	{ setMode(PROPERTY_IDENTIFIER_MODE); }
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 305
IS	:
	{ currentMode == STRING_MODE }?=>
	'is'
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 309
NOT	:
	{ currentMode == STRING_MODE }?=>
	'not'
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 313
NULL	:
	{ currentMode == STRING_MODE }?=>
	'null'
	;				
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 317
IN	:
	{ currentMode == STRING_MODE }?=>
	'in'
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 321
LEFT	:
	{ currentMode == STRING_MODE }?=>
	'left'
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 325
RIGHT	:
	{ currentMode == STRING_MODE }?=>
	'right'
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 329
OUTER	:
	{ currentMode == STRING_MODE }?=>
	'outer'
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 333
JOIN	:
	{ currentMode == STRING_MODE }?=>
	'join'
	{ setMode(CLASS_IDENTIFIER_MODE); }
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 338
ON	:
	{ currentMode == STRING_MODE }?=>
	'on'
	{ setMode(PROPERTY_IDENTIFIER_MODE); }
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 343
ORDER	:
	{ currentMode == STRING_MODE }?=>
	'order'
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 347
BY	:
	{ currentMode == STRING_MODE }?=>
	'by'
	{ setMode(PROPERTY_IDENTIFIER_MODE); }
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 352
ASC_DESC
	:
	{ currentMode == STRING_MODE }?=>
	'asc' | 'desc'
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 357
LEFT_PAREN
	:
	'('
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 361
RIGHT_PAREN
	:
	')'
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 365
MAPPED_PARAM
	:	
	':'(LETTER|DIGIT|'_')+
	{ setMode(STRING_MODE);  } 
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 370
CLASS_IDENTIFIER
	:
	{ currentMode == CLASS_IDENTIFIER_MODE }?=>
	IDENTIFIER
	{ setMode(STRING_MODE);  } 
	;		
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 376
PROPERTY_IDENTIFIER
	: 
	{ currentMode == PROPERTY_IDENTIFIER_MODE }?=>
	IDENTIFIER
	{ setMode(STRING_MODE);  }
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 382
ASTERISK
	:
	'*'
	{ setMode(STRING_MODE);  }
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 387
WS  	:
	(' '|'\r'|'\t'|'\u000C'|'\n') 
	{ 
		$channel=HIDDEN; 
	}
    	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 393
fragment IDENTIFIER
	:
	LETTER(LETTER|DIGIT|'.'|'_')+
	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 397
fragment DIGIT
	:
	'0'..'9'
    	;
// $ANTLR src "T:\transfer-root\antlr/Tql.g" 401
fragment LETTER
	: 
	'a'..'z'
	;
